
/* -------------------------------------------------------------------------- */

/*

macro_tclwrap.c

United States Naval Observatory
USNO/NRL Optical Interferometer
3450 Massachusetts Avenue, NW
Washington, DC  20392-5420

Subsystem:
----------
Metrology data reduction

Description:
------------
This file contains tcl wrapper functions (written in C) which call simple math
macros in the macro.h header file.

Functions:
----------
Macro_TclWrappers, Tcl_MAXDATA, Tcl_MINDATA, Tcl_MUSIGMA.

Modification history:
---------------------
1994 Apr 13 - Nicholas Elias, USNO/AD5
              File created with function Tcl_MUSIGMA( ).
1994 May 07 - Nicholas Elias, USNO/AD5
              Functions Tcl_MAXDATA( ) and Tcl_MINDATA( ) added.
1995 Sep 15 - Nicholas Elias, USNO/AD5
              Function Macro_TclWrappers( ) added.

*/

/* -------------------------------------------------------------------------- */

/* Include files */

#include <stdio.h>            /* Standard I/O */
#include <stdlib.h>           /* Standard library */
#include <math.h>             /* Math */
#include <string.h>           /* String */
#include <malloc.h>           /* Memory allocation */

#include "tcl.h"              /* tcl */

#include "macro.h"            /* Math macros */
#include "macro_tclwrap.h"    /* Math macro tcl wrapper prototypes */

/* -------------------------------------------------------------------------- */

/*

Macro_TclWrappers

Description:
------------
This function registers macros from the macro.h file into tcl.

Inputs:
-------
*pmtInterp - This is the pointer to the Tcl_Interp variable which contains the
	     tcl interpreter.

Outputs:
--------
*pmtInterp - This is the pointer to the Tcl_Interp variable which contains the
	     tcl interpreter.

Modification history:
---------------------
1995 Sep 15 - Nicholas Elias, USNO/AD5
	      Function created.

*/

/* -------------------------------------------------------------------------- */

int Macro_TclWrappers( Tcl_Interp *pmtInterp )

{

  /* Register the C functions as tcl commands */
  
  Tcl_CreateCommand( pmtInterp, "maxdata", Tcl_MAXDATA, (ClientData*) NULL,
      (Tcl_CmdDeleteProc*) NULL );
  Tcl_CreateCommand( pmtInterp, "mindata", Tcl_MINDATA, (ClientData*) NULL,
      (Tcl_CmdDeleteProc*) NULL );
  Tcl_CreateCommand( pmtInterp, "musigma", Tcl_MUSIGMA, (ClientData*) NULL,
      (Tcl_CmdDeleteProc*) NULL );
      
  
  /* Return 0 to the calling function */
  
  return( 0 );

}

/* -------------------------------------------------------------------------- */

/*

Tcl_MAXDATA

Description:
------------
This function creates a tcl command which finds the maximum value of a numeric
vector.  It must be registered with a tcl (or tk or [incr] tcl) int main( ) 
function to be used.

All decimal typedefs are acceptable for the input list, but the maximum is
returned as a double.

Inputs:
-------
mtClientData - This is a ClientData structure used to pass a one-word value to 
               this command procedure (not actually used in this procedure).
iArgC        - This is the int variable which contains the number of arguments 
               (including the maxdata command itself) on the tcl command line.
**asArgV     - This is the pointer to the 2D char array which contains the tcl 
               command line arguments (including the maxdata command itself).
               asArgV[0] contains "maxdata" and asArgV[1] contains the list of 
               data.
               
Outputs:
--------
*pmtInterp - This is the pointer to the Tcl_Interp structure which contains the
             tcl interpreter information.

Modification history:
---------------------
1994 May 07 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int Tcl_MAXDATA( ClientData mtClientData, Tcl_Interp *pmtInterp, int iArgC, 
    char **asArgV )

{

  /* Declare the local variables */
  
  int iData;        /* This is the int variable which contains the data
                       counter */
  int iNumData;     /* This is the int variable which contains the number of
                       data */
  int iResult;      /* This is the int variable which contains the result of
                       functions */
  
  double dfDMax;    /* This is the double variable which contains the maximum
                       value of the double vector */
  
  double *pdfData;  /* This is the pointer to the 1D double array which contains
                       the data vector */
  
  char psTemp[30];  /* This is the pointer to the 1D char array which contains a
                       temporary string */
  
  char **asTclList; /* This is the pointer to the 2D char array which contains
                       the tcl list */
  
  
  /* Check to see if the number of arguments is correct */
  
  if ( iArgC != 2 )
  {
    pmtInterp->result = "!! Tcl_MAXDATA: Wrong number of arguments...";
    return( TCL_OK );
  }
  
  
  /* Split the list and determine the number of elements */
  
  iResult = Tcl_SplitList( pmtInterp, asArgV[1], &iNumData, &asTclList );
  
  if ( iResult == TCL_ERROR )
  {
    pmtInterp->result = "!! Tcl_MAXDATA:  Not enough memory...";
    return( TCL_OK );
  }
 
  pdfData = malloc( iNumData * sizeof( double ) );
  
  for ( iData = 0; iData <= ( iNumData - 1 ); iData++ )
  {
    iResult = Tcl_GetDouble( pmtInterp, asTclList[iData], &pdfData[iData] );
    if ( iResult == TCL_ERROR )
    {
      pmtInterp->result = "!! Tcl_MAXDATA: Invalid element in data vector...";
      free( (char*) asTclList );
      free( (char*) pdfData );
      return( TCL_OK );
    }
  }
  
  
  /* Find the maximum value of the double vector */
 
  MAXDATA( iNumData, &pdfData[0], dfDMax );
  
  
  /* Deallocate the memory */
  
  free( (char*) asTclList );
  free( (char*) pdfData );
  
  
  /* Return dfDMax and TCL_OK to the calling function */
  
  sprintf( psTemp, "%f", dfDMax );
  Tcl_AppendElement( pmtInterp, psTemp );
  
  return( TCL_OK );
  
}

/* -------------------------------------------------------------------------- */

/*

Tcl_MINDATA

Description:
------------
This function creates a tcl command which finds the minimum value of a numeric
vector.  It must be registered with a tcl (or tk or [incr] tcl) int main( ) 
function to be used.

All decimal typedefs are acceptable for the input list, but the minimum is
returned as a double.

Inputs:
-------
mtClientData - This is a ClientData structure used to pass a one-word value to 
               this command procedure (not actually used in this procedure).
iArgC        - This is the int variable which contains the number of arguments 
               (including the mindata command itself) on the tcl command line.
**asArgV     - This is the pointer to the 2D char array which contains the tcl 
               command line arguments (including the mindata command itself).
               asArgV[0] contains "mindata" and asArgV[1] contains the list of 
               data.
               
Outputs:
--------
*pmtInterp - This is the pointer to the Tcl_Interp structure which contains the
             tcl interpreter information.

Modification history:
---------------------
1994 May 07 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int Tcl_MINDATA( ClientData mtClientData, Tcl_Interp *pmtInterp, int iArgC, 
    char **asArgV )

{

  /* Declare the local variables */
  
  int iData;        /* This is the int variable which contains the data
                       counter */
  int iNumData;     /* This is the int variable which contains the number of
                       data */
  int iResult;      /* This is the int variable which contains the result of
                       functions */
  
  double dfDMin;    /* This is the double variable which contains the minimum
                       value of the double vector */
  
  double *pdfData;  /* This is the pointer to the 1D double array which contains
                       the data vector */
  
  char psTemp[30];  /* This is the pointer to the 1D char array which contains a
                       temporary string */
  
  char **asTclList; /* This is the pointer to the 2D char array which contains
                       the tcl list */
  
  
  /* Check to see if the number of arguments is correct */
  
  if ( iArgC != 2 )
  {
    pmtInterp->result = "!! Tcl_MINDATA: Wrong number of arguments...";
    return( TCL_OK );
  }
  
  
  /* Split the list and determine the number of elements */
  
  iResult = Tcl_SplitList( pmtInterp, asArgV[1], &iNumData, &asTclList );
  
  if ( iResult == TCL_ERROR )
  {
    pmtInterp->result = "!! Tcl_MINDATA: Not enough memory...";
    return( TCL_OK );
  }
 
  pdfData = malloc( iNumData * sizeof( double ) );
  
  for ( iData = 0; iData <= ( iNumData - 1 ); iData++ )
  {
    iResult = Tcl_GetDouble( pmtInterp, asTclList[iData], &pdfData[iData] );
    if ( iResult == TCL_ERROR )
    {
      pmtInterp->result = "!! Tcl_MINDATA: Invalid element in data vector...";
      free( (char*) asTclList );
      free( (char*) pdfData );
      return( TCL_OK );
    }
  }
  
  
  /* Find the minimum value of the double vector */

  MINDATA( iNumData, &pdfData[0], dfDMin );
  
  
  /* Deallocate the memory */
  
  free( (char*) asTclList );
  free( (char*) pdfData );
  
  
  /* Return dfDMin and TCL_OK to the calling function */
  
  sprintf( psTemp, "%f", dfDMin );
  Tcl_AppendElement( pmtInterp, psTemp );
  
  return( TCL_OK );
  
}

/* -------------------------------------------------------------------------- */

/*

Tcl_MUSIGMA

Description:
------------
This function creates a tcl command that calculates the average of a data vector
and its error.  It must be registered with a tcl (or tk or [incr] tcl) int 
main( ) function to be used.

Inputs:
-------
mtClientData - This is a ClientData structure used to pass a one-word value to 
               this command procedure (not actually used in this procedure).
iArgC        - This is the int variable which contains the number of arguments 
               (including the musigma command itself) on the tcl command line.
**asArgV     - This is the pointer to the 2D char array which contains the tcl 
               command line arguments (including the musigma command itself).
               asArgV[0] contains "musigma" and asArgV[1] contains the list of
               data.
Outputs:
--------
*pmtInterp - This is the pointer to the Tcl_Interp structure which contains the
             tcl interpreter information.

Modification history:
---------------------
1994 Apr 13 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int Tcl_MUSIGMA( ClientData mtClientData, Tcl_Interp *pmtInterp, int iArgC, 
    char **asArgV )

{

  /* Declare the local variables */
  
  int iData;          /* This is the int variable which contains the data
                         counter */
  int iNumData;       /* This is the int variable which contains the number of
                         data */
  int iResult;        /* This is the int variable which contains the result of
                         functions */
  
  double dfDataMu;    /* This is the double variable which contains the data
                         vector average */
  double dfDataSigma; /* This is the double variable which contains the error of
                         the data vector average */
  
  double *pdfData;    /* This is the pointer to the 1D double array which
                         contains the data vector */
  
  char psTemp[30];    /* This is the pointer to the 1D char array which contains
                         a temporary string */
  
  char **asTclList;   /* This is the pointer to the 2D char array which contains
                         the tcl list */
  
  
  /* Check to see if the number of arguments is correct */
  
  if ( iArgC != 2 )
  {
    pmtInterp->result = "!! Tcl_MUSIGMA: Wrong number of arguments...";
    return( TCL_OK );
  }
  
  
  /* Split the list and determine the number of elements */
  
  iResult = Tcl_SplitList( pmtInterp, asArgV[1], &iNumData, &asTclList );
  
  if ( iResult == TCL_ERROR )
  {
    pmtInterp->result = "!! Tcl_MUSIGMA:  Not enough memory...";
    return( TCL_OK );
  }

  if ( iNumData <= 1 )
  {
    pmtInterp->result = "!! Tcl_MUSIGMA:  Number of data must be > 1 ...";
    return( TCL_OK );
  }

  pdfData = malloc( iNumData * sizeof( double ) );
  
  for ( iData = 0; iData <= ( iNumData - 1 ); iData++ )
  {
    iResult = Tcl_GetDouble( pmtInterp, asTclList[iData], &pdfData[iData] );
    if ( iResult == TCL_ERROR )
    {
      pmtInterp->result = "!! Tcl_MUSIGMA: Invalid element in data vector...";
      free( (char*) asTclList );
      free( (char*) pdfData );
      return( TCL_OK );
    }
  }
  
  
  /* Calculate the average and its error */

  MUSIGMA( iNumData, &pdfData[0], dfDataMu, dfDataSigma );
  
  if ( dfDataSigma  == -1.0 )
  {
    pmtInterp->result = "!! Tcl_MUSIGMA: Function ended unsuccessfully...";
    free( (char*) asTclList );
    free( (char*) pdfData );
    return( TCL_OK );
  }
  
  
  /* Deallocate the memory */
  
  free( (char*) asTclList );
  free( (char*) pdfData );
  
  
  /* Return iNumData, dfDataMu, dfDataSigma, and TCL_OK to the calling 
     function */
  
  sprintf( psTemp, "%d", iNumData );
  Tcl_AppendElement( pmtInterp, psTemp );
  
  sprintf( psTemp, "%f", dfDataMu );
  Tcl_AppendElement( pmtInterp, psTemp );
  
  sprintf( psTemp, "%f", dfDataSigma );
  Tcl_AppendElement( pmtInterp, psTemp );
  
  return( TCL_OK );
  
}

