
/* -------------------------------------------------------------------------- */

/*

memory2.c

USNO/NRL Optical Interferometer
United States Naval Observatory
3450 Massachusetts Avenue, NW
Washington, DC 20392-5420

Subsystem:
----------
My software libraries

Description:
------------
This file contains functions which allocate memory.

Functions:
----------
ALLOC1_double, ALLOC2_double, ALLOC3_double, ALLOC4_double, ALLOC5_double,
DEALLOC1_double, DEALLOC2_double, DEALLOC3_double, DEALLOC4_double,
DEALLOC5_double.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              File created with functions ALLOC1_double( ), ALLOC2_double( ),
              ALLOC3_double( ), ALLOC4_double( ), and ALLOC5_double( ).
1995 May 28 - Nicholas Elias, USNO/AD5
              Functions DEALLOC1_double( ), DEALLOC2_double( ),
              DEALLOC3_double( ), DEALLOC4_double( ), and DEALLOC5_double( )
              added.
1997 May 17 - Nicholas Elias, USNO/AD5
              File memory.c renamed to memory2.c.

*/

/* -------------------------------------------------------------------------- */

/* Includes */

#include <stdio.h>   /* Standard I/O */

#include "memory2.h" /* Memory allocation macros */
#include "message.h" /* Message function prototypes */

/* -------------------------------------------------------------------------- */

/*

ALLOC1_double

Description:
------------
This function allocates memory for a one-dimensional double array.

Inputs:
-------
**adfVar - This is the pointer to the 1D double array.
iNumRow  - This is the int variable which contains the number of rows.

Outputs:
--------
**adfVar - This is the pointer to the 1D double array.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int ALLOC1_double( double **adfVar, int iNumRow )

{

  /* Allocate the memory */

  ALLOC1( (*adfVar), double, iNumRow );
  
  if ( (*adfVar) == NULL )
  {
    NrWarning( "!! ALLOC1_double: Memory allocation problem..." );
    return( -1 );
  }
  else
  {
    return( 0 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

ALLOC2_double

Description:
------------
This function allocates memory for a two-dimensional double array.

Inputs:
-------
***adfVar - This is the pointer to the 2D double array.
iNumRow   - This is the int variable which contains the number of rows.
iNumCol   - This is the int variable which contains the number of columns.

Outputs:
--------
***adfVar - This is the pointer to the 2D double array.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int ALLOC2_double( double ***adfVar, int iNumRow, int iNumCol )

{

  /* Allocate the memory */

  ALLOC2( (*adfVar), double, iNumRow, iNumCol );
  
  if ( (*adfVar) == NULL )
  {
    NrWarning( "!! ALLOC2_double: Memory allocation problem..." );
    return( -1 );
  }
  else
  {
    return( 0 );
  }

}

/* -------------------------------------------------------------------------- */

/*

ALLOC3_double

Description:
------------
This function allocates memory for a three-dimensional double array.

Inputs:
-------
****adfVar - This is the pointer to the 3D double array.
iNumRow    - This is the int variable which contains the number of rows.
iNumCol    - This is the int variable which contains the number of columns.
iNumDep    - This is the int variable which contains the number of depths.

Outputs:
--------
****adfVar - This is the pointer to the 3D double array.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int ALLOC3_double( double ****adfVar, int iNumRow, int iNumCol, int iNumDep )

{

  /* Allocate the memory */

  ALLOC3( (*adfVar), double, iNumRow, iNumCol, iNumDep );
  
  if ( (*adfVar) == NULL )
  {
    NrWarning( "!! ALLOC3_double: Memory allocation problem..." );
    return( -1 );
  }
  else
  {
    return( 0 );
  }

}

/* -------------------------------------------------------------------------- */

/*

ALLOC4_double

Description:
------------
This function allocates memory for a four-dimensional double array.

Inputs:
-------
*****adfVar - This is the pointer to the 4D double array.
iNumRow     - This is the int variable which contains the number of rows.
iNumCol     - This is the int variable which contains the number of columns.
iNumDep1    - This is the int variable which contains the number of depths (1).
iNumDep2    - This is the int variable which contains the number of depths (2).

Outputs:
--------
*****adfVar - This is the pointer to the 4D double array.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int ALLOC4_double( double *****adfVar, int iNumRow, int iNumCol, int iNumDep1,
    int iNumDep2 )

{

  /* Allocate the memory */

  ALLOC4( (*adfVar), double, iNumRow, iNumCol, iNumDep1, iNumDep2 );
  
  if ( (*adfVar) == NULL )
  {
    NrWarning( "!! ALLOC4_double: Memory allocation problem..." );
    return( -1 );
  }
  else
  {
    return( 0 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

ALLOC5_double

Description:
------------
This function allocates memory for a five-dimensional double array.

Inputs:
-------
******adfVar - This is the pointer to the 5D double array.
iNumRow      - This is the int variable which contains the number of rows.
iNumCol      - This is the int variable which contains the number of columns.
iNumDep1     - This is the int variable which contains the number of depths (1).
iNumDep2     - This is the int variable which contains the number of depths (2).
iNumDep3     - This is the int variable which contains the number of depths (3).

Outputs:
--------
******adfVar - This is the pointer to the 5D double array.

Modification history:
---------------------
1995 May 19 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int ALLOC5_double( double ******adfVar, int iNumRow, int iNumCol, int iNumDep1,
    int iNumDep2, int iNumDep3 )

{

  /* Allocate the memory */

  ALLOC5( (*adfVar), double, iNumRow, iNumCol, iNumDep1, iNumDep2, iNumDep3 );
  
  if ( (*adfVar) == NULL )
  {
    NrWarning( "!! ALLOC5_double: Memory allocation problem..." );
    return( -1 );
  }
  else
  {
    return( 0 );
  }
   
}

/* -------------------------------------------------------------------------- */

/*

DEALLOC1_double

Description:
------------
This function deallocates memory for a one-dimensional double array.

Inputs:
-------
**adfVar - This is the pointer to the 1D double array.

Outputs:
--------
**adfVar - This is the pointer to the 1D double array.

Modification history:
---------------------
1995 May 28 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int DEALLOC1_double( double **adfVar )

{

  /* Deallocate the memory */

  if ( (*adfVar) != NULL )
  {
    DEALLOC1( (*adfVar) );
    return( 0 );
  }
  else
  {
    NrWarning( "!! DEALLOC1_double: Memory deallocation problem..." );
    return( -1 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

DEALLOC2_double

Description:
------------
This function deallocates memory for a two-dimensional double array.

Inputs:
-------
***adfVar - This is the pointer to the 2D double array.

Outputs:
--------
***adfVar - This is the pointer to the 2D double array.

Modification history:
---------------------
1995 May 28 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int DEALLOC2_double( double ***adfVar )

{

  /* Deallocate the memory */

  if ( (*adfVar) != NULL )
  {
    DEALLOC2( (*adfVar) );
    return( 0 );
  }
  else
  {
    NrWarning( "!! DEALLOC2_double: Memory deallocation problem..." );
    return( -1 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

DEALLOC3_double

Description:
------------
This function deallocates memory for a three-dimensional double array.

Inputs:
-------
****adfVar - This is the pointer to the 3D double array.

Outputs:
--------
****adfVar - This is the pointer to the 3D double array.

Modification history:
---------------------
1995 May 28 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int DEALLOC3_double( double ****adfVar )

{

  /* Deallocate the memory */

  if ( (*adfVar) != NULL )
  {
    DEALLOC3( (*adfVar) );
    return( 0 );
  }
  else
  {
    NrWarning( "!! DEALLOC3_double: Memory deallocation problem..." );
    return( -1 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

DEALLOC4_double

Description:
------------
This function deallocates memory for a four-dimensional double array.

Inputs:
-------
*****adfVar - This is the pointer to the 4D double array.

Outputs:
--------
*****adfVar - This is the pointer to the 4D double array.

Modification history:
---------------------
1995 May 28 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int DEALLOC4_double( double *****adfVar )

{

  /* Deallocate the memory */

  if ( (*adfVar) != NULL )
  {
    DEALLOC4( (*adfVar) );
    return( 0 );
  }
  else
  {
    NrWarning( "!! DEALLOC4_double: Memory deallocation problem..." );
    return( -1 );
  }
  
}

/* -------------------------------------------------------------------------- */

/*

DEALLOC5_double

Description:
------------
This function deallocates memory for a five-dimensional double array.

Inputs:
-------
******adfVar - This is the pointer to the 5D double array.

Outputs:
--------
******adfVar - This is the pointer to the 5D double array.

Modification history:
---------------------
1995 May 28 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* -------------------------------------------------------------------------- */

int DEALLOC5_double( double ******adfVar )

{

  /* Deallocate the memory */

  if ( (*adfVar) != NULL )
  {
    DEALLOC5( (*adfVar) );
    return( 0 );
  }
  else
  {
    NrWarning( "!! DEALLOC5_double: Memory deallocation problem..." );
    return( -1 );
  }
  
}

